package com.cnbm.packing.controller;

import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.cnbm.admin.annotation.LogOperation;
import com.cnbm.common.constant.Constant;
import com.cnbm.common.page.PageData;
import com.cnbm.common.utils.ExcelUtils;
import com.cnbm.common.utils.Result;
import com.cnbm.common.validator.AssertUtils;
import com.cnbm.common.validator.ValidatorUtils;
import com.cnbm.common.validator.group.AddGroup;
import com.cnbm.common.validator.group.DefaultGroup;
import com.cnbm.common.validator.group.UpdateGroup;
import com.cnbm.packing.dto.IdVo;
import com.cnbm.packing.dto.WoPackagingBoxDTO;
import com.cnbm.packing.entity.WoPackagingBox;
import com.cnbm.packing.entity.WoPackagingBoxSubstrate;
import com.cnbm.packing.excel.WoPackagingBoxExcel;
import com.cnbm.packing.excel.WoPackagingBoxSubstrateExcel;
import com.cnbm.packing.mapper.WoPackagingBoxMapper;
import com.cnbm.packing.service.WoPackagingBoxServiceBiz;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiImplicitParam;
import io.swagger.annotations.ApiImplicitParams;
import io.swagger.annotations.ApiOperation;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.security.access.prepost.PreAuthorize;
import org.springframework.web.bind.annotation.*;
import springfox.documentation.annotations.ApiIgnore;

import javax.servlet.http.HttpServletResponse;
import java.time.LocalDateTime;
import java.util.List;
import java.util.Map;


/**
 * 包装箱表  前端控制器
 *
 * @author packingGenerator
 * @since  2023-02-15
 */
@RestController
@RequestMapping("/packing/woPackagingBox")
@Api(tags="包装箱表")
public class WoPackagingBoxController {
    @Autowired
    private WoPackagingBoxServiceBiz woPackagingBoxService;
    @Autowired
    private WoPackagingBoxMapper woPackagingBoxMapper;

    @GetMapping("page")
    @ApiOperation("分页")
    @ApiImplicitParams({
        @ApiImplicitParam(name = Constant.PAGE, value = "当前页码，从1开始", paramType = "query", required = true, dataTypeClass=Integer.class) ,
        @ApiImplicitParam(name = Constant.LIMIT, value = "每页显示记录数", paramType = "query",required = true, dataTypeClass=Integer.class) ,
        @ApiImplicitParam(name = Constant.ORDER_FIELD, value = "排序字段", paramType = "query", dataTypeClass=String.class) ,
        @ApiImplicitParam(name = Constant.ORDER, value = "排序方式，可选值(asc、desc)", paramType = "query", dataTypeClass=String.class),
        @ApiImplicitParam(name = "startTime", value = "开始时间", paramType = "query", dataTypeClass = LocalDateTime.class),
        @ApiImplicitParam(name = "endTime", value = "结束时间", paramType = "query", dataTypeClass = LocalDateTime.class),
        @ApiImplicitParam(name = "boxNo", value = "boxid", paramType = "query", dataTypeClass = String.class),
//        @ApiImplicitParam(name = "lineBody", value = "线体", paramType = "query", dataTypeClass = Integer.class),
//        @ApiImplicitParam(name = "printStatus", value = "打印状态：0、未打印，1、已打印", paramType = "query", dataTypeClass = Integer.class),
        @ApiImplicitParam(name = "model", value = "模式，1-手动模式；2-自动模式", paramType = "query", dataTypeClass = Integer.class)
    })
    @PreAuthorize("@ex.hasAuthority('packing:woPackagingBox:page')")
    public Result<PageData<WoPackagingBoxDTO>> page(@ApiIgnore @RequestParam Map<String, Object> params){
        PageData<WoPackagingBoxDTO> page = woPackagingBoxService.page(params);

        return new Result<PageData<WoPackagingBoxDTO>>().ok(page);
    }

    @GetMapping("{id}")
    @ApiOperation("信息")
    @PreAuthorize("@ex.hasAuthority('packing:woPackagingBox:info')")
    public Result<WoPackagingBoxDTO> get(@PathVariable("id") Long id){
        WoPackagingBoxDTO data = woPackagingBoxService.get(id);

        return new Result<WoPackagingBoxDTO>().ok(data);
    }

    @PostMapping
    @ApiOperation("保存")
    @LogOperation("保存")
    @PreAuthorize("@ex.hasAuthority('packing:woPackagingBox:save')")
    public IdVo save(@RequestBody WoPackagingBoxDTO dto){
        //效验数据
        ValidatorUtils.validateEntity(dto, AddGroup.class, DefaultGroup.class);

        return woPackagingBoxService.add(dto);
    }

    @PutMapping
    @ApiOperation("修改")
    @LogOperation("修改")
    @PreAuthorize("@ex.hasAuthority('packing:woPackagingBox:update')")
    public Result<Long> update(@RequestBody WoPackagingBoxDTO dto){
        //效验数据
        ValidatorUtils.validateEntity(dto, UpdateGroup.class, DefaultGroup.class);

        woPackagingBoxService.update(dto);

        return new Result<Long>().ok(dto.getId());
    }

    @DeleteMapping
    @ApiOperation("删除")
    @LogOperation("删除")
    @PreAuthorize("@ex.hasAuthority('packing:woPackagingBox:delete')")
    public Result delete(@RequestBody Long[] ids){
        //效验数据
        AssertUtils.isArrayEmpty(ids, "id");

        woPackagingBoxService.delete(ids);

        return new Result();
    }

    @GetMapping("export")
    @ApiOperation("导出")
    @LogOperation("导出")
    @PreAuthorize("@ex.hasAuthority('packing:woPackagingBox:export')")
    public void export(@ApiIgnore @RequestParam Map<String, Object> params, HttpServletResponse response) throws Exception {
        List<WoPackagingBoxDTO> list = woPackagingBoxService.list(params);

        ExcelUtils.exportExcelToTarget(response, null, list, WoPackagingBoxExcel.class);
    }

    @PostMapping(value = "list")
    @ApiOperation(value = "获取列表")
    public List<WoPackagingBoxDTO> list() { return woPackagingBoxService.list(); }

    @PostMapping(value = "printList/{id}")
    @ApiOperation(value = "包装箱打印列表")
    public Result<WoPackagingBoxDTO> printList(@PathVariable("id") Long id) {

        WoPackagingBoxDTO data = woPackagingBoxService.printList(id);

        return new Result<WoPackagingBoxDTO>().ok(data);
    }

    @GetMapping("exportSubstrateList")
    @ApiOperation("箱单明细导出")
    @LogOperation("箱单明细导出")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "startTime", value = "开始时间", paramType = "query", dataTypeClass = LocalDateTime.class),
            @ApiImplicitParam(name = "endTime", value = "结束时间", paramType = "query", dataTypeClass = LocalDateTime.class),
            @ApiImplicitParam(name = "boxNo", value = "boxid", paramType = "query", dataTypeClass = String.class),
//            @ApiImplicitParam(name = "lineBody", value = "线体", paramType = "query", dataTypeClass = Integer.class),
//            @ApiImplicitParam(name = "printStatus", value = "打印状态：0、未打印，1、已打印", paramType = "query", dataTypeClass = Integer.class),
            @ApiImplicitParam(name = "model", value = "模式，1-手动模式；2-自动模式", paramType = "query", dataTypeClass = Integer.class)
    })
    public void exportSubstrateList(@ApiIgnore @RequestParam Map<String, Object> params, HttpServletResponse response) throws Exception {
        List<WoPackagingBoxSubstrate> list = woPackagingBoxService.substrateList(params);
        if (list.size() > 0) {
            ExcelUtils.exportExcelToTarget(response, "箱单明细", list, WoPackagingBoxSubstrateExcel.class);
        }
    }

    @GetMapping("boxList/{woSubstrateId}")
    @ApiOperation("查询模组id所在包装箱")
    public List<WoPackagingBox> boxList(@PathVariable("woSubstrateId") String woSubstrateId){
        return woPackagingBoxService.boxList(woSubstrateId);
    }

    @PostMapping("insertSubstrateManual")
    @ApiOperation("手动新增包装箱")
    @LogOperation("手动新增包装箱")
    public Result<IdVo> insertSubstrateManual(@RequestBody WoPackagingBoxDTO dto){

        //验证包装箱是否重名
        QueryWrapper<WoPackagingBox> wrapper = new QueryWrapper<>();
        wrapper.eq(WoPackagingBox.BOX_NO, dto.getBoxNo());
        if(woPackagingBoxMapper.selectCount(wrapper)!= 0){
            return new Result().error(1,"BoxID已存在,请重新输入");
        }
        else {
            return new Result<IdVo>().ok(woPackagingBoxService.insertSubstrateManual(dto));
        }

    }

    @PostMapping("editSubstrateManual")
    @ApiOperation("手动编辑包装箱")
    @LogOperation("手动编辑包装箱")
    public Result editSubstrateManual(@RequestBody WoPackagingBoxDTO dto){

        woPackagingBoxService.editSubstrateManual(dto);

        return new Result();

    }

}